package vc.thinker.absctacts.protocolproxy.dto;

import vc.thinker.absctacts.protocolproxy.enums.DataType;
import vc.thinker.absctacts.protocolproxy.enums.service.IServiceParamDefine;

import java.util.Map;

/**
 * 服务执行命令
 *
 * @author HeTongHao
 * @since 2021/7/9 17:08
 */
public interface IServiceExecuteCmd {
    /**
     * 流水号id
     *
     * @return 流水号id
     */
    String getRequestId();

    /**
     * 设备唯一code
     *
     * @return 设备唯一code
     */
    String getDeviceCode();

    /**
     * 服务编号
     *
     * @return 服务编号
     */
    String getServiceCode();

    /**
     * 参数集合
     *
     * @return 参数集合
     */
    Map<String, Object> getParams();

    /**
     * 参数集合
     *
     * @param paramDefine 参数定义
     * @return 参数集合
     */
    default <T> T getParamValue(IServiceParamDefine paramDefine) {
        Object result = this.getParams().get(paramDefine.getCode());
        if (result instanceof String) {
            if (DataType.Int.equals(paramDefine.getDataType())) {
                result = Integer.valueOf((String) result);
            } else if (DataType.Float.equals(paramDefine.getDataType())) {
                result = Float.valueOf((String) result);
            }
        }
        return (T) result;
    }
}
